import BN from 'bn.js';
declare class WrappedCall {
    call: Uint8Array;
    callIndex: string;
    contractId: string;
    encrypted: boolean;
    origin: string;
    originPublicKey: Uint8Array;
    shardId: number;
    constructor(config: {
        call: Uint8Array;
        callIndex: string;
        contractId: string;
        encrypted: boolean;
        origin: string;
        shardId: number;
    });
}
declare class Call {
    transaction_action: 'create_account' | 'call' | 'transfer' | 'view_method_call' | 'deploy';
    receiver: string;
    amount: BN | null;
    wasm_file: string | null;
    method: string | null;
    args: string | null;
    to: string | null;
    constructor(config: {
        transaction_action: 'create_account' | 'call' | 'transfer' | 'view_method_call' | 'deploy';
        receiver: string;
        amount: BN | null;
        wasm_file: string | null;
        method: string | null;
        args: string | null;
        to: string | null;
    });
}
declare class Calls {
    ops: Call[];
    constructor(config: {
        ops: Call[];
    });
}
declare class Outcome {
    view_result_log: string[];
    view_result: Uint8Array;
    outcome_logs: string[];
    outcome_receipt_ids: Uint8Array[];
    outcome_gas_burnt: BN;
    outcome_token_burnt: BN;
    outcome_executor_id: string;
    outcome_status: Uint8Array | null;
    constructor(config: {
        view_result_log: string[];
        view_result: Uint8Array;
        outcome_logs: string[];
        outcome_receipt_ids: Uint8Array[];
        outcome_gas_burnt: BN;
        outcome_token_burnt: BN;
        outcome_executor_id: string;
        outcome_status: Uint8Array | null;
    });
}
declare class WrappedOutcome {
    outcome: Uint8Array;
    callIndex: string;
    contractId: string;
    encrypted: boolean;
    origin: string;
    shardId: number;
    constructor(config: {
        outcome: Uint8Array;
        callIndex: string;
        contractId: string;
        encrypted: boolean;
        origin: string;
        shardId: number;
    });
}
declare class Outcomes {
    ops: Outcome[];
    state_root: Uint8Array;
    constructor(config: {
        ops: Outcome[];
        state_root: Uint8Array;
    });
}
declare const buildCall: (call: Call) => Uint8Array;
declare const buildCalls: (calls: Calls) => Uint8Array;
declare const parseCall: (buf: Uint8Array) => Call;
declare const parseCalls: (buf: Uint8Array) => Calls;
declare const buildOutcome: (outcome: Outcome) => Uint8Array;
declare const buildOutcomes: (outcomes: Outcomes) => Uint8Array;
declare const parseOutcome: (buf: Uint8Array) => Outcome;
declare const parseOutcomes: (buf: Uint8Array) => Outcomes;
export { WrappedCall, Call, Calls, buildCall, parseCall, buildCalls, parseCalls, WrappedOutcome, Outcome, Outcomes, buildOutcome, parseOutcome, buildOutcomes, parseOutcomes };
