// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { deserialize, serialize } from 'borsh';
/* eslint-disable sort-keys, camelcase */

class WrappedCall {
  constructor(config) {
    this.call = void 0;
    this.callIndex = void 0;
    this.contractId = void 0;
    this.encrypted = void 0;
    this.origin = void 0;
    this.originPublicKey = void 0;
    this.shardId = void 0;
    this.call = config.call;
    this.callIndex = config.callIndex;
    this.contractId = config.contractId;
    this.encrypted = config.encrypted;
    this.origin = config.origin;
    this.shardId = config.shardId;
  }

}

class Call {
  constructor(config) {
    this.transaction_action = void 0;
    this.receiver = void 0;
    this.amount = void 0;
    this.wasm_file = void 0;
    this.method = void 0;
    this.args = void 0;
    this.to = void 0;
    this.transaction_action = config.transaction_action;
    this.receiver = config.receiver;
    this.amount = config.amount;
    this.wasm_file = config.wasm_file;
    this.method = config.method;
    this.args = config.args;
    this.to = config.to;
  }

}

class Calls {
  constructor(config) {
    this.ops = void 0;
    this.ops = config.ops;
  }

}

const callSchema = new Map([[Call, {
  kind: 'struct',
  fields: [['transaction_action', 'string'], ['receiver', 'string'], ['amount', {
    kind: 'option',
    type: 'u128'
  }], ['wasm_file', {
    kind: 'option',
    type: 'string'
  }], ['method', {
    kind: 'option',
    type: 'string'
  }], ['args', {
    kind: 'option',
    type: 'string'
  }], ['to', {
    kind: 'option',
    type: 'string'
  }]]
}]]);
const callsSchema = new Map();
callsSchema.set(Calls, {
  kind: 'struct',
  fields: [['ops', [Call]]]
});
callsSchema.set(Call, callSchema.get(Call));

class Outcome {
  constructor(config) {
    this.view_result_log = void 0;
    this.view_result = void 0;
    this.outcome_logs = void 0;
    this.outcome_receipt_ids = void 0;
    this.outcome_gas_burnt = void 0;
    this.outcome_token_burnt = void 0;
    this.outcome_executor_id = void 0;
    this.outcome_status = void 0;
    this.view_result_log = config.view_result_log;
    this.view_result = config.view_result;
    this.outcome_logs = config.outcome_logs;
    this.outcome_receipt_ids = config.outcome_receipt_ids;
    this.outcome_gas_burnt = config.outcome_gas_burnt;
    this.outcome_token_burnt = config.outcome_token_burnt;
    this.outcome_executor_id = config.outcome_executor_id;
    this.outcome_status = config.outcome_status;
  }

}

class WrappedOutcome {
  constructor(config) {
    this.outcome = void 0;
    this.callIndex = void 0;
    this.contractId = void 0;
    this.encrypted = void 0;
    this.origin = void 0;
    this.shardId = void 0;
    this.outcome = config.outcome;
    this.callIndex = config.callIndex;
    this.contractId = config.contractId;
    this.encrypted = config.encrypted;
    this.origin = config.origin;
    this.shardId = config.shardId;
  }

}

class Outcomes {
  constructor(config) {
    this.ops = void 0;
    this.state_root = void 0;
    this.ops = config.ops;
    this.state_root = config.state_root;
  }

}

const outcomeSchema = new Map([[Outcome, {
  kind: 'struct',
  fields: [['view_result_log', ['string']], ['view_result', ['u8']], ['outcome_logs', ['string']], ['outcome_receipt_ids', [['u8', 32]]], ['outcome_gas_burnt', 'u64'], ['outcome_token_burnt', 'u128'], ['outcome_executor_id', 'String'], ['outcome_status', {
    kind: 'option',
    type: ['u8']
  }]]
}]]);
const outcomesSchema = new Map();
outcomesSchema.set(Outcomes, {
  kind: 'struct',
  fields: [['ops', [Outcome]], ['state_root', ['u8', 32]]]
});
outcomesSchema.set(Outcome, outcomeSchema.get(Outcome));

const buildCall = call => {
  const buf = serialize(callSchema, call);
  return buf;
};

const buildCalls = calls => {
  if (calls.ops.length === 0) {
    return new Uint8Array(0);
  }

  const buf = serialize(callsSchema, calls);
  return buf;
};

const parseCall = buf => {
  return deserialize(callSchema, Call, Buffer.from(buf));
};

const parseCalls = buf => {
  if (buf.length === 0) {
    return {
      ops: []
    };
  }

  return deserialize(callsSchema, Calls, Buffer.from(buf));
};

const buildOutcome = outcome => {
  const buf = serialize(outcomeSchema, outcome);
  return buf;
};

const buildOutcomes = outcomes => {
  const buf = serialize(outcomesSchema, outcomes);
  return buf;
};

const parseOutcome = buf => {
  return deserialize(outcomeSchema, Outcome, Buffer.from(buf));
};

const parseOutcomes = buf => {
  return deserialize(outcomesSchema, Outcomes, Buffer.from(buf));
};

export { WrappedCall, Call, Calls, buildCall, parseCall, buildCalls, parseCalls, WrappedOutcome, Outcome, Outcomes, buildOutcome, parseOutcome, buildOutcomes, parseOutcomes };