"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Contract = void 0;

var _api = require("@polkadot/api");

var _utilCrypto = require("@polkadot/util-crypto");

var _crypto = require("@skyekiwi/crypto");

var _driver = require("@skyekiwi/driver");

var _util = require("@skyekiwi/util");

var _borsh = require("./borsh.cjs");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
class Contract {
  static buildInitialState(calls) {
    return (0, _borsh.buildCalls)(calls);
  }

  static parseInitialState(encodedCalls) {
    return (0, _borsh.parseCalls)(encodedCalls);
  }

  static intoSecretContract(calls, wasmBlob) {
    return {
      initialState: Contract.buildInitialState(calls),
      wasmBlob: wasmBlob
    };
  }

  static async upstream(registry, contract) {
    const mnemonic = process.env.SEED_PHRASE;

    if (!(0, _utilCrypto.mnemonicValidate)(mnemonic)) {
      throw new Error('mnemonic failed to read - e2e.spec.ts');
    }

    if (contract.initialState.length !== 0) {
      const provider = new _api.WsProvider('wss://staging.rpc.skye.kiwi');
      const api = await _api.ApiPromise.create({
        provider: provider
      });
      const encryptionSchema = new _crypto.EncryptionSchema();
      const sksRaw = await api.query.registry.secretKeepers();
      const sks = sksRaw.toJSON();
      const sealer = new _crypto.DefaultSealer();
      sealer.unlock((0, _utilCrypto.mnemonicToMiniSecret)(process.env.SEED_PHRASE));

      if (!sks || sks.length === 0) {
        console.warn('no secret keepers found. Testing mode only.');
        encryptionSchema.addMember(sealer.getAuthorKey());
      } else {
        for (const sk of sks) {
          const pk = (await api.query.registry.publicKey(sk)).toString();
          encryptionSchema.addMember((0, _util.hexToU8a)(pk));
        }
      }

      await provider.disconnect();
      return await _driver.Driver.upstreamContract(registry, contract, sealer, encryptionSchema);
    } else {
      return await _driver.Driver.upstreamContract(registry, contract);
    }
  }

  static async downstream(registry, secretId) {
    const mnemonic = process.env.SEED_PHRASE;

    if (!(0, _utilCrypto.mnemonicValidate)(mnemonic)) {
      throw new Error('mnemonic failed to read - e2e.spec.ts');
    }

    const sealer = new _crypto.DefaultSealer();
    sealer.unlock((0, _utilCrypto.mnemonicToMiniSecret)(mnemonic));
    return await _driver.Driver.downstreamContract(secretId, registry, [(0, _utilCrypto.mnemonicToMiniSecret)(mnemonic)], sealer);
  }

}

exports.Contract = Contract;