"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Storage = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _fs = _interopRequireDefault(require("fs"));

var _crypto = require("@skyekiwi/crypto");

var _ipfs = require("@skyekiwi/ipfs");

var _util = require("@skyekiwi/util");

// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
// import {Driver} from '@skyekiwi/driver';
class CallInfo {
  constructor(rawCall, isArchive, isEncrypted, origin, originPublicKey, contractIndex) {
    this.rawCall = rawCall;
    this.isArchive = isArchive;
    this.isEncrypted = isEncrypted;
    this.origin = origin;
    this.originPublicKey = originPublicKey;
    this.contractIndex = contractIndex;
    this.rawOutcome = void 0;
    this.rawOutcome = undefined;
  }

  writeOutcome(outcome) {
    this.rawOutcome = outcome;
  }

}

class ContractInfo {
  constructor(contractIndex) {
    this.contractIndex = contractIndex;
    this.wasmBlob = void 0;
    this.homeShard = void 0;
    this.metadataCID = void 0;
    this.metadata = void 0;
    this.isInitialStateDecrypted = void 0;
    this.isInitialStateDecrypted = false;
  }

  async init(api) {
    const ipfs = new _ipfs.IPFS();

    try {
      this.homeShard = Number((await api.query.secrets.homeShard(this.contractIndex)).toString());
      const wasmCID = (await api.query.secrets.wasmBlob(this.contractIndex)).toString();
      const content = await ipfs.cat(wasmCID);
      this.wasmBlob = (0, _util.hexToU8a)(content);
      this.metadataCID = (await api.query.secrets.metadata(this.contractIndex)).toString();

      if (this.metadataCID !== '0000000000000000000000000000000000000000000000') {
        // the contract has no private initial state
        this.isInitialStateDecrypted = true;
      }
    } catch (e) {
      console.log(e);
    }
  }

}

class ShardInfo {
  constructor(shardId) {
    this.shardId = shardId;
    this.calls = void 0;
    this.highRemoteCallIndex = void 0;
    this.highLocalCallIndex = void 0;
    this.highRemoteSyncedBlockIndex = void 0;
    this.highRemoteConfirmedBlockIndex = void 0;
    this.confirmationThreshold = void 0;
    this.calls = [];
    this.highRemoteCallIndex = 0;
    this.highLocalCallIndex = 0;
    this.highRemoteSyncedBlockIndex = 0;
    this.highRemoteConfirmedBlockIndex = 0;
    this.confirmationThreshold = 0;
  }

  async init(api) {
    const calls = (await api.query.sContract.callHistory(this.shardId)).toJSON();
    console.log(calls);
  }

}

var _key = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("key");

class Storage {
  constructor(api, key, path) {
    this.api = api;
    this.contracts = void 0;
    this.calls = void 0;
    this.shards = void 0;
    Object.defineProperty(this, _key, {
      writable: true,
      value: void 0
    });
    (0, _classPrivateFieldLooseBase2.default)(this, _key)[_key] = key;

    if (path) {
      this.fromFile(path);
    } else {
      this.contracts = {};
      this.calls = {};
      this.shards = {};
    }
  }

  fromFile(path) {
    const source = _fs.default.readFileSync(path);

    const decrypted = _crypto.SymmetricEncryption.decrypt(source, (0, _classPrivateFieldLooseBase2.default)(this, _key)[_key]);

    const obj = JSON.parse((0, _util.u8aToString)(decrypted));
    this.contracts = obj.contracts;
    this.calls = obj.calls;
    this.shards = obj.shards;
  }

  saveToFile(path) {
    const obj = (0, _util.stringToU8a)(JSON.stringify(this));

    const encrypted = _crypto.SymmetricEncryption.encrypt(obj, (0, _classPrivateFieldLooseBase2.default)(this, _key)[_key]);

    _fs.default.writeFileSync(path, encrypted);
  }

}

exports.Storage = Storage;