import { ApiPromise } from '@polkadot/api';
import { Call, Outcome } from './borsh';
declare class CallInfo {
    rawCall: Call;
    isArchive: boolean;
    isEncrypted: boolean;
    origin: string;
    originPublicKey: string;
    contractIndex: number;
    rawOutcome: Outcome | undefined;
    constructor(rawCall: Call, isArchive: boolean, isEncrypted: boolean, origin: string, originPublicKey: string, contractIndex: number);
    writeOutcome(outcome: Outcome): void;
}
declare class ContractInfo {
    contractIndex: number;
    wasmBlob: Uint8Array;
    homeShard: number;
    metadataCID: string;
    metadata: Uint8Array;
    isInitialStateDecrypted: boolean;
    constructor(contractIndex: number);
    init(api: ApiPromise): Promise<void>;
}
declare class ShardInfo {
    shardId: number;
    calls: number[];
    highRemoteCallIndex: number;
    highLocalCallIndex: number;
    highRemoteSyncedBlockIndex: number;
    highRemoteConfirmedBlockIndex: number;
    confirmationThreshold: number;
    constructor(shardId: number);
    init(api: ApiPromise): Promise<void>;
}
export declare class Storage {
    #private;
    api: ApiPromise;
    contracts: {
        [contractIndex: number]: ContractInfo;
    };
    calls: {
        [callIndex: number]: CallInfo;
    };
    shards: {
        [shardId: number]: ShardInfo;
    };
    constructor(api: ApiPromise, key: Uint8Array, path?: string);
    fromFile(path: string): void;
    saveToFile(path: string): void;
}
export {};
