import _classPrivateFieldLooseBase from "@babel/runtime/helpers/esm/classPrivateFieldLooseBase";
import _classPrivateFieldLooseKey from "@babel/runtime/helpers/esm/classPrivateFieldLooseKey";
// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import fs from 'fs';
import { SymmetricEncryption } from '@skyekiwi/crypto';
import { IPFS } from '@skyekiwi/ipfs';
import { hexToU8a, stringToU8a, u8aToString } from '@skyekiwi/util';

// import {Driver} from '@skyekiwi/driver';
class CallInfo {
  constructor(rawCall, isArchive, isEncrypted, origin, originPublicKey, contractIndex) {
    this.rawCall = rawCall;
    this.isArchive = isArchive;
    this.isEncrypted = isEncrypted;
    this.origin = origin;
    this.originPublicKey = originPublicKey;
    this.contractIndex = contractIndex;
    this.rawOutcome = void 0;
    this.rawOutcome = undefined;
  }

  writeOutcome(outcome) {
    this.rawOutcome = outcome;
  }

}

class ContractInfo {
  constructor(contractIndex) {
    this.contractIndex = contractIndex;
    this.wasmBlob = void 0;
    this.homeShard = void 0;
    this.metadataCID = void 0;
    this.metadata = void 0;
    this.isInitialStateDecrypted = void 0;
    this.isInitialStateDecrypted = false;
  }

  async init(api) {
    const ipfs = new IPFS();

    try {
      this.homeShard = Number((await api.query.secrets.homeShard(this.contractIndex)).toString());
      const wasmCID = (await api.query.secrets.wasmBlob(this.contractIndex)).toString();
      const content = await ipfs.cat(wasmCID);
      this.wasmBlob = hexToU8a(content);
      this.metadataCID = (await api.query.secrets.metadata(this.contractIndex)).toString();

      if (this.metadataCID !== '0000000000000000000000000000000000000000000000') {
        // the contract has no private initial state
        this.isInitialStateDecrypted = true;
      }
    } catch (e) {
      console.log(e);
    }
  }

}

class ShardInfo {
  constructor(shardId) {
    this.shardId = shardId;
    this.calls = void 0;
    this.highRemoteCallIndex = void 0;
    this.highLocalCallIndex = void 0;
    this.highRemoteSyncedBlockIndex = void 0;
    this.highRemoteConfirmedBlockIndex = void 0;
    this.confirmationThreshold = void 0;
    this.calls = [];
    this.highRemoteCallIndex = 0;
    this.highLocalCallIndex = 0;
    this.highRemoteSyncedBlockIndex = 0;
    this.highRemoteConfirmedBlockIndex = 0;
    this.confirmationThreshold = 0;
  }

  async init(api) {
    const calls = (await api.query.sContract.callHistory(this.shardId)).toJSON();
    console.log(calls);
  }

}

var _key = /*#__PURE__*/_classPrivateFieldLooseKey("key");

export class Storage {
  constructor(api, key, path) {
    this.api = api;
    this.contracts = void 0;
    this.calls = void 0;
    this.shards = void 0;
    Object.defineProperty(this, _key, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldLooseBase(this, _key)[_key] = key;

    if (path) {
      this.fromFile(path);
    } else {
      this.contracts = {};
      this.calls = {};
      this.shards = {};
    }
  }

  fromFile(path) {
    const source = fs.readFileSync(path);
    const decrypted = SymmetricEncryption.decrypt(source, _classPrivateFieldLooseBase(this, _key)[_key]);
    const obj = JSON.parse(u8aToString(decrypted));
    this.contracts = obj.contracts;
    this.calls = obj.calls;
    this.shards = obj.shards;
  }

  saveToFile(path) {
    const obj = stringToU8a(JSON.stringify(this));
    const encrypted = SymmetricEncryption.encrypt(obj, _classPrivateFieldLooseBase(this, _key)[_key]);
    fs.writeFileSync(path, encrypted);
  }

}